/**
 * 
 */

 import https from 'https';
 import crypto from 'crypto';

 import {Config, DATATYPE, DEFAULTEVENTID, SDKTYPE, PEOPLE_PROPERTY_ID} from './config';
 import People from './people';
 import ABTest from './abtest';


 // 只能有一个实例
 const DATracker = {
    config: {},
    people: null,
    abtest: null,
    /**
      * 
      * @param {String} appkey 应用appkey
      * @param {Object} config 自定义配置
    */
    init(appkey, config) {
        if (typeof appkey == 'undefined') {
            throw new Error("appkey不能为空");
        }
        const defaultConfig = Object.assign({}, Config, {
            appKey: '',
            ipPattern: '^(([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\.){3}([01]?\\d\\d?|2[0-4]\\d|25[0-5])$',
            superProperties: {}
        });
        this.config = Object.assign({}, defaultConfig, config || {});
        
        this.setConfig('appKey', appkey);
        this.people = People.init(this);
        this.abtest = ABTest.init(this);
        return this;
    },
    // type: warn, error, log
    log(type, msg) {
        if (this.getConfig('DEBUG')) {
            if (console[type]) {
                console[type](msg);
            }
        }
    },
    setConfig(name, value) {
        this.config[name] = value;
    },
    getConfig(name) {
        return this.config[name];
    },
    // 发送日志到服务器
    send(properties) {
        const sha1 = (str) => {
           return crypto.createHash('sha1').update(str).digest('hex').toUpperCase();
        };
        let dataStr = null;
        let base64Str = null;
        let encodeStr = null;
        if (typeof properties == null) {
            this.log('error',"上传日志不能为空");
        }
        try {
            dataStr = JSON.stringify(properties);
            base64Str = Buffer.from(dataStr).toString('base64');
            encodeStr = encodeURIComponent(base64Str);
            this.log('log', dataStr);
            let options = {
                hostname: this.getConfig('api_host'),
                port: '',
                path: this.getConfig('track_url') + '?data=' + encodeStr,
                method: 'GET',
                headers: {
                    'X-CLIENT-IP': properties.ip,
                    'X-SHA1-APPKEY': sha1(this.getConfig('appKey'))
                }
            };

            if (!properties.ip) {
                delete options.headers['X-CLIENT-IP'];
            }

            options.agent = new https.Agent({
                keepAlive: true,
                timeout: 300000,
                keepAliveMsecs: 300000,
                // 每个主机允许的套接字的最大数量
                maxSockets: 100,
                //maxFreeSockets: 1000
            });

            let req = https.request(options, (res) => {});
            req.on('response', (res) => {
                if (res.statusCode != 200) {
                    this.log('error', 'Hubble Server 日志发送失败：'+ res.statusCode, dataStr);
                }
            });
            req.on('error', (e) => {
                this.log('warn', 'https request 失败 '+ e, );
            });
            req.end();
        } catch (error) {
            this.log('error', "日志字符串化失败 "+ error);
        }
    },
    // 检测公共属性ip是否合法
    checkIp(ip) {
        let reg = new RegExp(this.getConfig('ipPattern'));
        return reg.test(ip);
    },
    // 检测用户id不能为空
    checkUserId(userId) {
        let bool = true;
        if (userId == null || typeof userId == 'undefined' || userId == '') {
            bool = false;
        }
        return bool;
    },
    // 检测事件id不能为空
    checkEventId(eventId) {
        let bool = true;
        if (eventId == null || typeof eventId == 'undefined' || eventId == '') {
            bool = false;
        }
        return bool;
    },
    // sdk默认事件属性
    getDefaultProperties() {
        return {
            sdkType: SDKTYPE,
            sdkVersion: this.getConfig('sdkVersion'),
            appKey: this.getConfig('appKey')
        };
    },
    /**
     * 上报事件日志（对外）
     * @param {String} userId  
     * @param {String} eventId 事件id 
     * @param {Object} properties 事件自定义属性
     * @param {Object} systemProperties 系统属性
     */
    track(userId, eventId, properties, systemProperties) {
        try {
            if (!this.checkUserId(userId)) {
                this.log('warn', "用户id不能为空");
                return;
            }
            if (!this.checkEventId(eventId)) {
                this.log('warn', "事件id不能为空");
                return;
            }
            // 事件类型
            let dataType = DATATYPE;
            // 如果是内置事件,事件类型重新设置
            if (DEFAULTEVENTID[eventId]) {
                dataType = DEFAULTEVENTID[eventId].dataType;
            }
            properties = typeof properties == 'object' ? properties : {};
            systemProperties = typeof systemProperties == 'object' ? systemProperties : {};
            
            // ip
            let ip = typeof systemProperties.ip == 'string' ? systemProperties.ip : null;
            // 事件耗时
            let costTime = typeof systemProperties.costTime == 'number' ? systemProperties.costTime : null;

            // 系统属性
            let event = {};
            event.dataType = dataType;
            event.time = new Date().getTime();
            event.userId = userId;
            event.eventId = eventId;
            
            if (costTime != null) {
                event.costTime = costTime;
            }
            if (ip != null) {
                event.ip = ip;
                if (!this.checkIp(ip)) {
                    this.log('warn', 'ip格式不对');
                }
            }
            // 合并公共用户自定义事件属性，排除用户事件
            if (eventId != PEOPLE_PROPERTY_ID) {
                let superProperties = this.getConfig('superProperties') || {};
                properties = Object.assign({}, superProperties, properties);
            }
            // 自定义属性
            event.attributes = properties;
            event = Object.assign({}, this.getDefaultProperties(), event);
            this.send(event);
        } catch (error) {
            this.log('error', error);
        }
    },
    // 用户行为事件的通用属性设置
    register_attributes(properties) {
        let superProperties = this.getConfig('superProperties') || {};
        superProperties = Object.assign({}, superProperties, properties || {});
        this.setConfig('superProperties', superProperties);
    },
    // 用户行为事件的通用属性设置（不覆盖之前的）
    register_attributes_once(properties) {
        let superProperties = this.getConfig('superProperties') || {};
        superProperties = Object.assign({}, properties || {}, superProperties);
        this.setConfig('superProperties', superProperties);
    },
    // 查看当前已设置的用户行为事件的通用属性
    current_attributes() {
        return this.getConfig('superProperties') || {};
    },
    // 删除一个用户行为事件的通用属性
    unregister_attributes(propertyName) {
        if (typeof propertyName === 'string') {
            let superProperties = this.getConfig('superProperties') || {};
            delete superProperties[propertyName];
            this.setConfig({'superProperties': superProperties});
        }
    },
    // 删除所有已设置的用户行为事件的通用属性
    clear_attributes() {
        this.setConfig('superProperties', {});
    },
    // 用户注册，把匿名ID和注册ID贯通起来
    // oldUserId: 用户匿名id
    // newUserId: 用户id
    signup(oldUserId, newUserId) {
        if (this.checkUserId(oldUserId) &&  this.checkUserId(newUserId)) {
            let properties = {
                'oldUserId': oldUserId,
                'newUserId': newUserId
            };
            this.track(newUserId, 'da_u_signup', properties);
        } else {
            this.log('warn', 'signup方法：两个参数不能为空！');
        }
    }
 };


 export default DATracker